package peer

import (
	"strconv"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type peerMongoAccessor struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
}

// New creates a new instance of the peerMongoAccessor
func New() *peerMongoAccessor {
	return &peerMongoAccessor{}
}

/*
* Nothing special here, just the basic CRUD operations
 */

func (wfa *peerMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return wfa.GenericDeleteOne(id, wfa)
}

func (wfa *peerMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	return wfa.GenericUpdateOne(set.(*Peer), id, wfa, &Peer{})
}

func (wfa *peerMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return wfa.GenericStoreOne(data.(*Peer), wfa)
}

func (wfa *peerMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return wfa.GenericStoreOne(data, wfa)
}

func (wfa *peerMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var peer Peer
	res_mongo, code, err := mongo.MONGOService.LoadOne(id, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}
	res_mongo.Decode(&peer)

	return &peer, 200, nil
}

func (wfa peerMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []Peer
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r)
	}
	return objs, 200, nil
}

func (wfa *peerMongoAccessor) Search(filters *dbs.Filters, search string) ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	if (filters == nil || len(filters.And) == 0 || len(filters.Or) == 0) && search != "" {
		s, err := strconv.Atoi(search)
		if err == nil {
			filters = &dbs.Filters{
				Or: map[string][]dbs.Filter{ // search by name if no filters are provided
					"state": {{Operator: dbs.EQUAL.String(), Value: s}},
				},
			}
		} else {
			filters = &dbs.Filters{
				Or: map[string][]dbs.Filter{ // search by name if no filters are provided
					"abstractobject.name": {{Operator: dbs.LIKE.String(), Value: search}},
					"url":                 {{Operator: dbs.LIKE.String(), Value: search}},
				},
			}
		}
	}
	res_mongo, code, err := mongo.MONGOService.Search(filters, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not store to db. Error: " + err.Error())
		return nil, code, err
	}
	var results []Peer
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r)
	}
	return objs, 200, nil
}
